!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.2
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
!
! !ROUTINE: get_saldas
!  \label{get_saldas}
!
! !REVISION HISTORY:
!  19 Sept 2008: Sujay Kumar: Initial Implementation
!
! !INTERFACE:
subroutine get_saldas(n,findex)
! !USES:
  use LIS_coreMod,        only : LIS_rc
  use LIS_timeMgrMod,     only : LIS_tick, LIS_get_nstep
  use LIS_logMod,         only : LIS_logunit, LIS_endrun
  use saldas_forcingMod,    only : saldas_struc

  implicit none
! !ARGUMENTS: 
  integer, intent(in) :: n
  integer, intent(in) :: findex
!  
! !DESCRIPTION:
!  Opens, reads, and interpolates 3-hrly, SALDAS forcing. 
!  The temporal frequency of the SALDAS data is 3 hours. 
!  At the beginning of a simulation, the code 
!  reads the most recent past data (nearest 3 hour interval), and
!  the nearest future data. These two datasets are used to 
!  temporally interpolate the data to the current model timestep. 
!  The strategy for missing data is to go backwards up to 10 days to get
!  forcing at the same time of day.
!
!  The arguments are: 
!  \begin{description}
!  \item[n]
!    index of the nest
!  \end{description}
!
!  The routines invoked are: 
!  \begin{description}
!  \item[LIS\_tick](\ref{LIS_tick}) \newline
!    determines the SALDAS data times
!  \item[saldasfile](\ref{saldasfile}) \newline
!    Puts together a time stamped filename for SALDAS data. 
!  \item[read\_saldas](\ref{read_saldas}) \newline
!    reads and spatially interpolates SALDAS data
!  \end{description}
!EOP

  integer :: t,f
  integer :: ferror
  integer :: try
  integer, parameter :: ndays = 10  ! # of days to look back for forcing data
  integer :: order     ! 1 indicates lesser interpolation boundary time
                       ! 2 indicates greater interpolation boundary time
  integer :: doy1, yr1, mo1, da1, hr1, mn1, ss1
  integer :: doy2, yr2, mo2, da2, hr2, mn2, ss2
  integer :: movetime  ! 1=move time2 into time1
  real*8  :: timenow, time1, time2
  real*8  :: dumbtime1, dumbtime2
  real    :: gmt1, gmt2,ts1,ts2
  character(len=80) :: name
  integer :: nstep

  saldas_struc(n)%findtime1=0
  saldas_struc(n)%findtime2=0
  movetime=0
  nstep = LIS_get_nstep(LIS_rc,n)

!-----------------------------------------------------------------
! Determine required SALDAS data times 
! (previous assimilation, current & future assimilation hours)
! The adjustment of the hour and the direction will be done
! in the subroutines that generate the names
!-----------------------------------------------------------------
  yr1=LIS_rc%yr    !Time now
  mo1=LIS_rc%mo
  da1=LIS_rc%da
  hr1=LIS_rc%hr
  mn1=LIS_rc%mn
  ss1=0
  ts1=0        
  call LIS_tick(timenow,doy1,gmt1,yr1,mo1,da1,hr1,mn1,ss1,ts1)
  
  yr1=LIS_rc%yr    !Previous Hour
  mo1=LIS_rc%mo
  da1=LIS_rc%da
  hr1=3*((LIS_rc%hr)/3)
  mn1=0
  ss1=0
  ts1=0
  call LIS_tick(time1,doy1,gmt1,yr1,mo1,da1,hr1,mn1,ss1,ts1)
  
  yr2=LIS_rc%yr    !Next Hour
  mo2=LIS_rc%mo
  da2=LIS_rc%da
  hr2=3*((LIS_rc%hr)/3)
  mn2=0
  ss2=0
  ts2=3*60*60
  call LIS_tick(time2,doy2,gmt2,yr2,mo2,da2,hr2,mn2,ss2,ts2)
  if(timenow.ge.saldas_struc(n)%saldastime2) then
     movetime=1
     saldas_struc(n)%findtime2=1
  endif
  
  if ( nstep.eq.0 .or. nstep.eq.1 .or.LIS_rc%rstflag(n).eq.1 ) then 
     saldas_struc(n)%findtime1=1
     saldas_struc(n)%findtime2=1
     movetime=0
        LIS_rc%rstflag(n) = 0
     endif
     LIS_rc%shortflag=2            !Time averaged SW
     LIS_rc%longflag=2             !Time averaged LW 
     
!-------------------------------------------------------------------
! Establish saldastime1
!-------------------------------------------------------------------
     if (saldas_struc(n)%findtime1==1) then  
        order=1   
        ferror = 0
        try = 0
        ts1 = -24*60*60
        do
           if ( ferror /= 0 ) then
              exit
           end if
           try = try+1
           call saldasfile(name,saldas_struc(n)%saldasdir,&
                yr1,mo1,da1,hr1,saldas_struc(n)%ncold)
           write(LIS_logunit,*) 'Reading ',trim(name)
!           call read_saldas(order,n,findex, name,ferror, try)
	   call read_saldas( order, n, findex, yr1, mo1, da1, hr1, ferror, name )
           if ( ferror == 1 ) then 
!-------------------------------------------------------------------
! successfully retrieved forcing data
!-------------------------------------------------------------------
              saldas_struc(n)%saldastime1=time1
           else  
!-------------------------------------------------------------------
! ferror still=0, so roll back one day
!-------------------------------------------------------------------
              call LIS_tick(dumbtime1,doy1,gmt1,yr1,mo1,da1,hr1,mn1,ss1,ts1)
           end if
           if ( try > ndays ) then 
              print *, 'ERROR: SALDAS data gap exceeds 10 days on file 1'
              call LIS_endrun
           end if
        end do
     endif
     if(movetime.eq.1) then
        saldas_struc(n)%saldastime1=saldas_struc(n)%saldastime2
        saldas_struc(n)%findtime2=1 
        do f=1,LIS_rc%met_nf(findex)
           do t=1,LIS_rc%ngrid(n)
              saldas_struc(n)%metdata1(f,t)=saldas_struc(n)%metdata2(f,t)
           enddo
        enddo
     endif
     if(saldas_struc(n)%findtime2.eq.1) then 
        order=2   
        ferror = 0
        try = 0
        ts2 = -24*60*60
        do
           if ( ferror /= 0 ) exit
           try = try+1
           call saldasfile(name,saldas_struc(n)%saldasdir,&
                yr2,mo2,da2,hr2,saldas_struc(n)%ncold)
           write(LIS_logunit,*) 'Reading ',trim(name)
!           call read_saldas(order,n,findex,name,ferror,try)
	   call read_saldas( order, n, findex, yr2, mo2, da2, hr2, ferror, name )
           if ( ferror == 1 ) then 
!-------------------------------------------------------------------
! successfully retrieved forcing data
!-------------------------------------------------------------------
              saldas_struc(n)%saldastime2=time2
           else  
!-------------------------------------------------------------------
! ferror still=0, so roll back one day
!-------------------------------------------------------------------
              call LIS_tick(dumbtime2,doy2,gmt2,yr2,mo2,da2,hr2,mn2,ss2,ts2)
           end if
           if ( try > ndays ) then 
              print *, 'ERROR: SALDAS data gap exceeds 10 days on file 2'
              call LIS_endrun
           end if
        end do
     endif
     
84   format('now',i4,4i3,2x,'pvt ',a22,' nxt ',a22)
!  if ((LIS_rc%gridchange==1).and.(saldas_struc(n)%ncold==288)) then
!     LIS_rc%gridchange=0
!  endif
  return 

end subroutine get_saldas


!BOP
! !ROUTINE: saldasfile
! \label{saldasfile}
!
! !INTERFACE:
subroutine saldasfile(name,saldasdir,yr,mo,da,hr,ncold)
  
  implicit none
! !ARGUMENTS: 
  character(len=*), intent(in)  :: saldasdir
  integer, intent(in)       :: yr,mo,da,hr,ncold
  character(len=*), intent(out) :: name
! !DESCRIPTION:
!   This subroutine puts together SALDAS file name for 
!   3 hour file intervals
! 
!  The arguments are:
!  \begin{description}
!  \item[name]
!   name of the timestamped SALDAS file
!  \item[saldasdir]
!    Name of the SALDAS directory
!  \item[yr]
!    year 
!  \item[mo]
!   month
!  \item[da]
!   day of month
!  \item[hr]
!   hour of day
!  \item[ncold]
!   Number of columns (used as to determine the SALDAS resolution)
!  \end{description}
!
!EOP
  integer uyr,umo,uda,uhr,i,c,ii,jj
  character(len=2) :: initcode
  character*1 fbase(80),fsubs(80)
  character*1 ftime(10),fdir(17)
  
  character(LEN=100) :: temp
  ii = ncold
  jj = 181

!-------------------------------------------------------------------  
! Make variables for the time used to create the file
! We don't want these variables being passed out
!-------------------------------------------------------------------
  uyr=yr
  umo=mo
  uda=da
  uhr = 3*(hr/3)  !hour needs to be a multiple of 3 hours
!-------------------------------------------------------------------
!  Determine initcode for the hour of the forecast file
!  If the time is 12 or later the file is time stamped
!  with the next day.  So check for that first
!-------------------------------------------------------------------

  if(uhr<3)then
     initcode = '00'   
  elseif(uhr<6)then
     initcode = '03'
  elseif(uhr<9)then
     initcode = '06'
  elseif(uhr<12)then
     initcode = '09'
  elseif(uhr<15)then
     initcode = '12'
  elseif(uhr<18)then
     initcode = '15'
  elseif(uhr<21)then
     initcode = '18'
  elseif(uhr<24)then
     initcode = '21'
  endif
  
  write(UNIT=temp,FMT='(A40)') saldasdir 
  read(UNIT=temp,FMT='(80A1)') (fbase(i),i=1,80)
  
  write(UNIT=temp,FMT='(a1,i4,i2,a10)') '/',uyr,umo,'/LIS_HIST_'
  read(UNIT=temp,FMT='(17A1)') fdir
  do i=1,17
     if(fdir(i).eq.(' ')) fdir(i)='0'
  enddo
  
  write(UNIT=temp,FMT='(i4,i2,i2,a2)') uyr,umo,uda,initcode
  read(UNIT=temp,FMT='(10A1)') ftime
  do i=1,10
     if(ftime(i).eq.(' ')) ftime(i)='0'
  enddo
  
  write(UNIT=temp,FMT='(A9)') '00.d01.nc'
  read(UNIT=temp,FMT='(80A1)') (fsubs(i),i=1,9)
  
  c=0
  do i=1,80
     if(fbase(i).eq.(' ').and.c.eq.0) c=i-1 
  enddo
  
  write(UNIT=temp,FMT='(80a1)') (fbase(i),i=1,c),(fdir(i),i=1,17), &
       (ftime(i),i=1,10),(fsubs(i),i=1,9)
  
  
  read(UNIT=temp, FMT='(a80)') name
  return

end subroutine saldasfile



 
