!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.2
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LIS_misc.h"
!BOP
!
! !ROUTINE: read_saldas 
!  \label{read_saldas}
!
! !REVISION HISTORY:
!  26 Jan 2007: Hiroko Beaudoing; Initial Specification adopted from 
!                                 LIS/retberg.F90
!  16 Feb 2016: Hiroko Beaudoing; Fixed indexes for precip and pressure fields 
!                                 so budget-bilinear interpolation applies to 
!                                 correct individual fields.
!  15 May 2017: Bailing Li; Added changes for reading in version 2.2 data
!                           that is in 3D array (4D in version 1 & 2).
!  22 Oct 2018: Daniel Sarmiento; Added changes to support version 3 data
!
! !INTERFACE:
subroutine read_saldas( order, n, findex, yr, mon, da, hr, ferror, name )

! !USES:
  use LIS_coreMod,          only : LIS_rc,LIS_domain, LIS_localPet, &
                                   LIS_masterproc
  use LIS_metforcingMod,    only : LIS_forc
  use LIS_timeMgrMod,       only : LIS_tick
  use LIS_logMod,           only : LIS_logunit, LIS_endrun
  use saldas_forcingMod, only : saldas_struc
  use LIS_forecastMod
#if (defined USE_NETCDF3 || defined USE_NETCDF4)
  use netcdf
#endif

  implicit none
! !ARGUMENTS:
  integer, intent(in)    :: order     ! lower(1) or upper(2) time interval bdry
  integer, intent(in)    :: n         ! nest
  integer, intent(in)    :: findex    ! forcing index
  integer, intent(in)    :: yr,mon,da,hr     ! data and hour (multiple of 3)
  integer, intent(inout) :: ferror           ! set to zero if there's an error
  character(len=*),  intent(in) :: name
!
! !DESCRIPTION:
!  For the given time, reads the parameters from 1 degree
!  SALDAS data, transforms into 9 LIS forcing 
!  parameters and interpolates to the LIS domain.
!
!  SALDAS variables used to force LIS are:
!  mean values starting at timestep, available every 3 hours \newline
!
!  NOTE-1: be aware that SALDAS has only total precipitation.
!  NOTE 2: only one wind component, it is magnitude \newline
!
!  SALDAS FORCING VARIABLES: \newline
!  1. TAS       Air Temperature [K] \newline
!  2. SHUM      Specific humidity [kg kg-1] \newline
!  3. DSWRF     Downward shortwave radiation [W m-2] \newline
!  4. DLWRF     Downward longwave radiation [W m-2] \newline
!  5. WIND      Wind speed [m s-1] \newline
!  6. PRES      Surface pressure [Pa] \newline
!  7. PRCP      Precipitation [kg m-2 s-1] \newline
!
!  \begin{description}
!  \item[order]
!    flag indicating which data to be read (order=1, read the previous 
!    3 hourly instance, order=2, read the next 3 hourly instance)
!  \item[n]
!    index of the nest
!  \item[yr]
!    current year
!  \item[mon]
!    current month
!  \item[da]
!    current day of the year
!  \item[hr]
!    current hour of day
!  \item[ferror]
!    flag to indicate success of the call (=0 indicates success)
!  \end{description}
! 
!  The routines invoked are: 
!  \begin{description}
!  \item[saldasgrid\_2\_lisgrid](\ref{saldasgrid_2_lisgrid}) \newline
!    transform the SALDAS data to the LIS grid 
!  \item[bilinear\_interp](\ref{bilinear_interp}) \newline
!    spatially interpolate the forcing data using bilinear interpolation
!  \item[conserv\_interp](\ref{conserv_interp}) \newline
!    spatially interpolate the forcing data using conservative interpolation
!  \end{description}
!
!EOP

! Specify Reanalysis SALDAS forcing parameters & file paths
  integer,parameter ::  nvars =  5          ! number of variables: last
                                             ! variable is of interest
  integer,parameter ::  ndims =  4          ! number of dimensions
  integer,parameter ::  z =      1
  real,   parameter ::  no_data_value = -9999. ! 2.e+20

  integer, parameter :: N_PF = 8 ! # of SALDAS forcing variables
  integer, parameter :: NF = 10   ! # of GLDAS forcing variables

!  character(10), dimension(N_PF), parameter :: saldas_fv = (/  &
!       'tas       ',    &
!       'shum      ',    & 
!       'dswrf     ',    &
!       'dlwrf     ',    &
!       'wind      ',    &
!       'pres      ',    &
!       'prcp      '     /)
  character(13), dimension(N_PF), parameter :: saldas_fv = (/  &
       'Tair_f_tavg  ',    &
       'Qair_f_tavg  ',    & 
       'SWdown_f_tavg',    &
       'LWdown_f_tavg',    &
       'Wind_f_tavg  ',    &
       'Psurf_f_tavg ',    &
       'Rainf_f_tavg ',    &
       'Snowf_tavg   '     /)

!  integer, dimension(N_PF), parameter :: fnum = (/ &
!       31, 33, 34, 35, 36, 37, 38 /) 

  character*100 :: infile

! netcdf variables
  integer :: ncid, varid, status

  integer :: mo
  real*8  :: timenow
  real    :: gmt
  integer :: doy,mn,ss,ts
  character :: cyr*4
  integer :: timestep

  integer :: i, j, v, ii, r, k, eindex, x, y
  integer :: ios                   ! set to non-zero if there's an error
  integer :: gldas, nsaldas     ! Size of I/O 1D fields
  integer :: iret, c
  real    :: gridDesco(50)         ! Input,output grid info arrays

  integer :: kk                    ! forecast index
  integer :: num_met               ! number of fields, V3 will have 6, 2 and 2.2 have 7 (precip)

  real,allocatable :: datain(:,:)  ! input data (longitude,latitude)
  real,allocatable :: temp2saldas(:,:,:)
  real,allocatable :: templdas(:,:,:)
  real,allocatable :: f(:)         ! 1D in fields
  real,allocatable :: go(:)        ! 1D out fields
  real,allocatable :: tg(:,:)      ! Interpolated 2D data field
  logical*1,allocatable :: lb(:)   ! input bitmap
  logical*1,allocatable :: lo(:)   ! output bitmaps
! following variables commented out since geogfill routine is not used
!  logical*1 :: geogmask(LIS_rc%lnc(n),LIS_rc%lnr(n))! 2D output bitmap
!  logical*1 :: tmask(LIS_rc%lnc(n),LIS_rc%lnr(n))   ! 2d valid temperature bitmap
! ______________________________

#if (defined USE_NETCDF3 || defined USE_NETCDF4)
   ! If a problem, ferror is set to zero
   ferror = 1
   mo = LIS_rc%lnc(n)*LIS_rc%lnr(n)
   num_met = 8

   ! Allocate memory
   allocate(datain(saldas_struc(n)%ncold,saldas_struc(n)%nrold))
   allocate(temp2saldas(saldas_struc(n)%ncold,saldas_struc(n)%nrold,NF))
   allocate(f(saldas_struc(n)%ncold*saldas_struc(n)%nrold))
   allocate(go(LIS_rc%lnc(n)*LIS_rc%lnr(n)))
   allocate(lb(saldas_struc(n)%ncold*saldas_struc(n)%nrold))
   allocate(lo(LIS_rc%lnc(n)*LIS_rc%lnr(n)))
   allocate(tg(LIS_rc%lnc(n),LIS_rc%lnr(n)))  
   allocate(templdas(LIS_rc%lnc(n),LIS_rc%lnr(n),LIS_rc%nf), stat=ios)
 
   if(ios .ne.0) then 
     write(LIS_logunit,*) '[ERR] Error allocating templdas,',LIS_localPet
     stop 344
   endif

   temp2saldas = no_data_value !0.0     ! initialize

  !----------------------------------------------------------------
  ! Establish which file timestep the date & hour correspond to
  ! and which file to open.
  !----------------------------------------------------------------
   mn=LIS_rc%mn ! Time of input file
   ss=0
   ts=0
   call LIS_tick(timenow,doy,gmt,yr,mon,da,hr,mn,ss,real(ts))

   ! One file per year--use day of year to get to the time record
   timestep = 8*(doy - 1) + (1 + hr/3)

   if(LIS_masterproc) then
      write(LIS_logunit,*)'[INFO] Order and month-timestep', order, timestep, doy, hr
   endif


!=== Open SALDAS forcing files ===

       status = nf90_open(name, nf90_NoWrite, ncid)
       if(status/=0) then
         if(LIS_masterproc) then 
            write(LIS_logunit,*)'[ERR] Problem opening file: ',name,status
            write(LIS_logunit,*)'[ERR]  Stopping...'
            call LIS_endrun
         endif
         call LIS_endrun
       else
         if(LIS_masterproc) write(LIS_logunit,*)'[INFO] Opened file: ',name
       end if

!      v=1
!      call modif(name,v,datain) 
!      print *, datain(150,150),saldas_struc(n)%ncold,saldas_struc(n)%nrold
!      STOP


     ! Forcing variable loop:
     do v = 1, num_met  ! Number of met fields in Saldas data

       ! Open netCDF file.
       status = nf90_inq_varid(ncid, trim(saldas_fv(v)), varid)
!       print *,v,varid
       status = nf90_get_var(ncid, varid, datain, &
                                     start=(/1,1/), &
       count=(/saldas_struc(n)%ncold,saldas_struc(n)%nrold/))
       status = nf90_get_var(ncid, varid, datain)

      !----------------------------------------------------------------
      ! Change data from SALDAS grid convention to LIS-domain
      ! Shift longitudes by 180deg. 
      !----------------------------------------------------------------
      ! call saldasgrid_2_lisgrid(saldas_struc(n)%ncold,&
      !                              saldas_struc(n)%nrold,datain)
       
!       print *, v, trim(saldas_fv(v)), varid, datain(250,150)
     ! Forcing variable loop:
      !call modif(name,v,datain) 

       
      !-----------------------------------------------------------------
      ! Filter out any unrealistic forcing values.
      ! Transfer SALDAS forcing fields to LIS format
      !-----------------------------------------------------------------
       do j=1,saldas_struc(n)%nrold
         do i=1,saldas_struc(n)%ncold
           
	   if ( datain(i,j) .ne. no_data_value ) then
           select case (v)
            case (1)! tair
              temp2saldas(i,j,1) = datain(i,j)
            case (2)! qair
              temp2saldas(i,j,2) = datain(i,j)
            case (3)! Shortwave
              if (datain(i,j) < 0.0001) then
                datain(i,j) = 0.0001
              endif
              temp2saldas(i,j,3) = datain(i,j)
            case (4)! Longwave
              temp2saldas(i,j,4) = datain(i,j)
            case (5)! Wind
              if (datain(i,j) < 0.0001) then
                datain(i,j) = 0.0001
              endif
              temp2saldas(i,j,5) = datain(i,j)  !Since absolute wind speed 
                                                   ! let U=WIND and V=0.0
            case (6)! pressure
              temp2saldas(i,j,7) = datain(i,j)/100   
            case (7)! Total precipitation: convective precp=0
              if (datain(i,j) < 0.0) then
                datain(i,j) = 0.0
              endif
              temp2saldas(i,j,8) = datain(i,j)
	    case (8)! SnowFall
              if (datain(i,j) < 0.0) then
                datain(i,j) = 0.0
              endif
              temp2saldas(i,j,9) = datain(i,j)
	    
           end select
	   
	   endif
	  ! if (v .eq. 1) temp2saldas(i,j,1) = datain(i,j)
	  ! if (v .eq. 2) temp2saldas(i,j,2) = datain(i,j)
         enddo
       enddo
     end do !v or variable loop
       ! Close netCDF file.
       status=nf90_close(ncid)
!
!       status = nf90_open(name, nf90_NoWrite, ncid)
!       if(status/=0) then
!         if(LIS_masterproc) then 
!            write(LIS_logunit,*)'[ERR] Problem opening file: ',name,status
!            write(LIS_logunit,*)'[ERR]  Stopping...'
!            call LIS_endrun
!         endif
!         call LIS_endrun
!       else
         if(LIS_masterproc) write(LIS_logunit,*)'[INFO] Opened file2: ',name
!       end if
!       status = nf90_inq_varid(ncid, "Tair_f_tavg", varid)
!       status = nf90_get_var(ncid, varid, datain)
!       print *, datain(:,10)
!       print *, "BEFORE DATAIN"
!       status = nf90_inq_varid(ncid, "Qair_f_tavg", varid)
!       status = nf90_get_var(ncid, varid, datain)
!       print *, datain(:,10)
!       print *, "after DATAIN"
!     print *, temp2saldas(:,50,1)
!     print *, " " 
!     print *, temp2saldas(:,50,2)
!     print *, " " 
!     print *, temp2saldas(:,50,3)
!     print *, " "
!     print *, LIS_rc%udef
     
!       status=nf90_close(ncid)

!     STOP
     

     !--------------------------------------------------------------
     ! Interpolate each forcing variable to LIS domain
     !--------------------------------------------------------------

     !=== Initialize input & output grid arrays
     gridDesco = 0
           
     !=== Set input & output grid array values (reanlSALDAS to LIS)
     gridDesco = LIS_rc%gridDesc(n,:)

     !=== Define input & output data bitmaps
     nsaldas = saldas_struc(n)%ncold*saldas_struc(n)%nrold
     gldas  = LIS_rc%lnc(n)*LIS_rc%lnr(n)

     !== valid value over land and ocean for Saldas data
     lb = .true.
     lo = .false.
   !  tmask = .false.

     templdas(:,:,9) = 0.0
     temp2saldas(:,:,9) = 0.0

     do v=1,NF-1      ! do not process convective precip field, which is set to 0
        if (v .ne. 6) then ! not the v-wind component, which is set to zero.
      
          !=== Transferring current data to 1-D array for interpolation
!          c=0
!          do i=1,saldas_struc(n)%nrold
!             do j=1,saldas_struc(n)%ncold
!                c = c + 1
!                f(c) = temp2saldas(j,i,v)
!          !      if(v.eq.1 .and. f(c).ne.-9999.0) write(LIS_logunit,*) c,f(c)
!             enddo
!          enddo

	    lb = .true.
            do r=1,saldas_struc(n)%nrold
              do c=1,saldas_struc(n)%ncold
                k= c+(r-1)*saldas_struc(n)%ncold
                f(k) = temp2saldas(c,r,v)
                if ( f(k) == -9999.0 ) then 
                  f(k)  = LIS_rc%udef
                  lb(k) = .false. 
                endif
!		print *,r,c,k,temp2saldas(c,r,v),v
              enddo
            enddo
!     print *, temp2saldas(:,50,1)
!     print *, " " 
!     print *, temp2saldas(:,50,2)
!     print *, " " 
!     print *, temp2saldas(:,50,3)
!     print *, " "
     
!      STOP

          !=== Interpolate if forcing and model grids are not both one deg.
          if( LIS_rc%gridDesc(n,9).eq.1.0 ) then

           select case( LIS_rc%met_interp(findex) )

             case( "bilinear" )
               call bilinear_interp(gridDesco,lb,f,lo,go,&
                  saldas_struc(n)%mi,mo, & 
                  LIS_domain(n)%lat, LIS_domain(n)%lon,&
                  saldas_struc(n)%w111,saldas_struc(n)%w121,&
                  saldas_struc(n)%w211,saldas_struc(n)%w221,& 
                  saldas_struc(n)%n111,saldas_struc(n)%n121,&
                  saldas_struc(n)%n211,saldas_struc(n)%n221,&
                  LIS_rc%udef, iret)

             case( "budget-bilinear" )
               if(v.eq.8) then 
                 call conserv_interp(gridDesco,lb,f,lo,go,&
                     saldas_struc(n)%mi,mo, & 
                     LIS_domain(n)%lat, LIS_domain(n)%lon,&
                     saldas_struc(n)%w112,saldas_struc(n)%w122,&
                     saldas_struc(n)%w212,saldas_struc(n)%w222,& 
                     saldas_struc(n)%n112,saldas_struc(n)%n122,&
                     saldas_struc(n)%n212,saldas_struc(n)%n222,&
                     LIS_rc%udef,iret)
               else
                 call bilinear_interp(gridDesco,lb,f,lo,go,saldas_struc(n)%mi,mo, & 
                     LIS_domain(n)%lat, LIS_domain(n)%lon,&
                     saldas_struc(n)%w111,saldas_struc(n)%w121,&
                     saldas_struc(n)%w211,saldas_struc(n)%w221, & 
                     saldas_struc(n)%n111,saldas_struc(n)%n121,&
                     saldas_struc(n)%n211,saldas_struc(n)%n221,LIS_rc%udef, iret)
               endif

           end select

         else ! forcing and model grids both one degree
            !k = 0
            !do r=1,LIS_rc%lnr(n)
            !   y = r + (LIS_rc%gridDesc(n,4) + 89.50) / 1.000
            !   do c=1,LIS_rc%lnc(n)
            !      x = c + (LIS_rc%gridDesc(n,5) + 179.50) / 1.000
            !      k = k + 1
            !      eindex = ((y - 1) * 360) + x
            !      go(k) = f(eindex)
            !      lo(k) = lb(eindex)
            !   end do
            !end do
            
!	    print *,"LGG GridDesc ", LIS_rc%gridDesc(n,4),LIS_rc%gridDesc(n,5)
!	    STOP
	    k = 0
            do r=1,LIS_rc%lnr(n)
!               y = r + (LIS_rc%gridDesc(n,4) + 55.975)/0.05
               y = r + (LIS_rc%gridDesc(n,4) + 55.9749985)/0.05
               do c=1,LIS_rc%lnc(n)
!                  x = c + (LIS_rc%gridDesc(n,5) + 80.975)/0.05
                  x = c + (LIS_rc%gridDesc(n,5) + 80.9749985)/0.05
                  k = k + 1
                  eindex = ((y - 1) * saldas_struc(n)%ncold ) + x
	    !	  eindex = k
                  go(k) = f(eindex)
                  lo(k) = lb(eindex)
	    !  print *, r,y,c,x, k, eindex
               end do
            end do
	    
         end if ! LIS_rc%domain 
    
         !=== Convert data to original 3D array & a 2D array to 
         !=== fill in of missing points due to geography difference  
!         tg = -9999.0
!         tg = 0.0 
          tg = LIS_rc%udef
         c = 0
         do j = 1, LIS_rc%lnr(n)
            do i = 1, LIS_rc%lnc(n)
!              write(LIS_logunit,*) i,j,gindex(i,j),lo(i+c)
              if(LIS_domain(n)%gindex(i,j).ne.-1) then 
               !geogmask(i,j) = lo(i+c)
                tg(i,j) = go(i+c)
              endif
            enddo
            c = c + LIS_rc%lnc(n)
         enddo

         !== no need to fill in for Saldas
         ! call geogfill2(n, LIS_rc%lnc(n),LIS_rc%lnr(n),geogmask,tg,v,tmask)
         !       write(LIS_logunit,*) gindex(21,3),tg(21,3),v
	 
	 call fillgaps_saldas(n,1,tg)

         do j = 1, LIS_rc%lnr(n)
           do i = 1, LIS_rc%lnc(n)
              if(LIS_domain(n)%gindex(i,j).ne.-1) then 
                !if ((tg(i,j) .lt. 0) .and. (tg(i,j) .ne. LIS_rc%udef).or.(tg(i,j) .eq. no_data_value) ) then
		if (tg(i,j) .eq. LIS_rc%udef)  then
                   !For version 3, replace the value with an undefined value instead of ending the run.
                   !For all other versions, keep old call to end the program.
!                   if(saldas_struc(n)%version == "3") then
                      write(LIS_logunit,*)'[WARN] No nearest neighbors, v, i, j',v,i,j,tg(i,j)
                      write(LIS_logunit,*)'[WARN] Check output to make sure the missing neighbor'
                      write(LIS_logunit,*)'is isolated and does not distort the output.'      
                    !  tg(i,j) = LIS_rc%udef ! New code to change data to undefined
		    !  tg(i,j) = tg(250,250)
		      write(LIS_logunit,*)tg(i,j)
!                   else
!                      write(LIS_logunit,*)'[WARN] No nearest neighbors, v, i, j',v,i,j,tg(i,j)
!                      call LIS_endrun()  ! Old code to call a fatal error
!                   endif
                endif
                templdas(i,j,v) = tg(i,j)
              endif
           end do ! c
        enddo    ! r

      else ! v==6, v-wind component, always zero
         templdas(:,:,6) = 0.0
	 temp2saldas(:,:,6) = 0.0
      endif
    enddo  ! end v=variable loop

    do v= 1,NF
      ! Fill in undefined and ocean points
      do r = 1,LIS_rc%lnr(n)
        do c = 1,LIS_rc%lnc(n)
          if (LIS_domain(n)%gindex(c,r).ne.-1) then
           if(order.eq.1)then
              saldas_struc(n)%metdata1(v,LIS_domain(n)%gindex(c,r))=templdas(c,r,v)
	      !saldas_struc(n)%metdata1(v,LIS_domain(n)%gindex(c,r))=temp2saldas(c,r,v)
           else
              saldas_struc(n)%metdata2(v,LIS_domain(n)%gindex(c,r))=templdas(c,r,v)
	      !saldas_struc(n)%metdata2(v,LIS_domain(n)%gindex(c,r))=temp2saldas(c,r,v)
           endif
          endif
        enddo !c
      enddo   !r
    enddo     !v

!  end do  ! End forecast member index loop
    open(10,file='test.bin',status='unknown',access='direct',recl=4)
      ii=0
      !   do c = 1,LIS_rc%lnc(n)
      do r = 1,LIS_rc%lnr(n)
        do c = 1,LIS_rc%lnc(n)

          ii=ii+1
	  !write(10,rec=ii)temp2saldas(c,r,1)
          write(10,rec=ii)templdas(c,r,1)
          !write(10,rec=ii)datain(c,r)
        enddo !c
      enddo   !r
    print *, LIS_rc%lnr(n), LIS_rc%lnc(n), saldas_struc(n)%ncold,saldas_struc(n)%nrold,&
          LIS_rc%gridDesc(n,4),LIS_rc%gridDesc(n,5)
!    print *, templdas(150:200,20,1),temp2saldas(150:200,20,1)
    CLOSE(10)
!    STOP

  ! Deallocate local interp-based variables:
  deallocate(datain)
  deallocate(temp2saldas)
  deallocate(f)
  deallocate(go)
  deallocate(lb)
  deallocate(lo)
  deallocate(tg)
  deallocate(templdas)
#endif

end subroutine read_saldas


!BOP
! 
! !ROUTINE: saldasgrid_2_lisgrid
! \label{saldasgrid_2_lisgrid}
!
! !REVISION HISTORY:
!  10 Apr 2002: Urszula Jambor;  Code adapted from 
!               ecmwfgrid_2_grid2catgrid, by R. Reichle
!  29 Jan 2007: Hiroko Kato; modified berggrid_2_lisgrid for Saldas data
!
! !INTERFACE:
subroutine saldasgrid_2_lisgrid( nx, ny, grid_data )
  use LIS_logMod,           only : LIS_logunit, LIS_endrun

  implicit none
! !ARGUMENTS:   
  integer, intent(in)                   :: nx, ny
  real, intent(inout), dimension(nx,ny) :: grid_data
!
! !DESCRIPTION:
! Changes grid data from SALDAS data convention to LIS convention
!
! SALDAS: Sorth-to-Nouth around Greenwich Meridian
! Global grid. Data are written in NetCDF from ``lower left to 
! upper right'' starting at 0.5-degree grid point center coordinates: 
! 0.5E,89.5S and going to 0.5W,89.5N.
! 
! LIS: South-to-North around Date Line
! Full global grid.  Starts at the southernmost latitude and date line, 
! going east and then north.
!
!EOP
  
  integer :: i, j, m, jc
  real    :: tmp, tmp_data1(nx)
  real, dimension(nx,ny) :: tmp_gus
  
  ! ------------------------------------------------------------------
  ! Some checks
  ! ------------------------------------------------------------------
!  if( ((nx /= 360) .or. (ny /= 180)) .and. ((nx /= 1440) .or. (ny /= 600)) ) then
!     write(LIS_logunit,*) '[ERR] Rsaldasgrid_2_gldasgrid(): This routine has only been'
!     write(LIS_logunit,*) '  checked for nx=360 and ny=180 (version 2 and 2.2) and for '
!     write(LIS_logunit,*) '  nx=1440 and ny=600 (version 3). ' 
!     write(LIS_logunit,*) '  Make sure you know what you are doing. STOPPING.'
!     call LIS_endrun()
!  end if  
!  if ((mod(nx,2) /= 0) .or. (mod(ny,2) /= 0)) then
!     write(LIS_logunit,*) '[ERR] Rsaldasgrid_2_gldasgrid(): This routine can only work'
!     write(LIS_logunit,*) '  for even nx and ny. Make sure you know'
!     write(LIS_logunit,*) '  what you are doing. STOPPING.'
!     call LIS_endrun()
!  end if
  
  !-------------------------------------------------------------------
  jc=0
  do j=ny,1,-1
    jc=jc+1
    do i=1,nx
      tmp_gus(i,jc) = grid_data(i,j)
    enddo
  enddo

  do j=1,ny
    do i=1,nx
      grid_data(i,j) = tmp_gus(i,j)
    enddo
  enddo
    
  
  
!  do j=1,ny
!     
!     do i=1,nx
!     tmp_data1(i)    = grid_data(i,j)
!     end do
!     
!     do i=1,nx/2
!
!        ! shift longitudes (wrapping around Greenwhich Meridian becomes
!        !  wrapping around Date Line)
!        m = i + nx/2
!        tmp          = tmp_data1(i)
!        tmp_data1(i) = tmp_data1(m)
!        tmp_data1(m) = tmp
!        
!     end do
     
!     do i=1,nx
!     grid_data(i,j) = tmp_data1(i)
!     end do
     
!  end do

end subroutine Saldasgrid_2_lisgrid

!subroutine modif(idatec,nvar,latic,latip,longc,longp,var_stepc,var_stepp)
subroutine modif(namep,nvar,var_stepc)
!==============================================================
! Example showing how to read/write a netcdf file in fortran90
!==============================================================

! !USES:
  use LIS_coreMod,          only : LIS_rc,LIS_domain, LIS_localPet, &
                                   LIS_masterproc
  use LIS_metforcingMod,    only : LIS_forc
  use LIS_timeMgrMod,       only : LIS_tick
  use LIS_logMod,           only : LIS_logunit, LIS_endrun
  use saldas_forcingMod, only : saldas_struc
  use LIS_forecastMod
USE netcdf

IMPLICIT NONE

INTEGER :: fidA, status, dimID_TIME, dimID_DEPTH, dimID_LAT, dimID_LON, &
&          mTIME, mDEPTH, mLAT, mLON, TIME_ID, DEPTH_ID, LAT_ID, LON_ID,&
&          TEMP_ID, fidM

character(LEN=*), INTENT(IN) :: namep

character(LEN=10) :: ndat
integer,INTENT(IN) :: nvar

CHARACTER(LEN=100) :: file_in, file_out

REAL*4,ALLOCATABLE,DIMENSION(:) :: TIME, DEPTH
REAL*4,ALLOCATABLE,DIMENSION(:,:) :: LAT, LON


!REAL*4,ALLOCATABLE,DIMENSION(:,:,:,:) :: TEMP
REAL*4,ALLOCATABLE,DIMENSION(:,:) :: TEMP
!real, INTENT(OUT) :: varout(1000,1360)
real :: varout(1000,1360)
integer, parameter :: xp=1360,yp=1000

!    INTEGER       , INTENT(IN   ) :: idatec (4) ! INTENT(IN    ) :: idatec(:)
!  REAL(KIND=4)       , INTENT(OUT  ) :: latic( xp, yp)
!  REAL(KIND=4)       , INTENT(OUT  ) :: latip( xp, yp)
!  REAL(KIND=4)       , INTENT(OUT  ) :: longc( xp, yp)
!  REAL(KIND=4)       , INTENT(OUT  ) :: longp( xp, yp)
  REAL(KIND=4)       , INTENT(OUT  ) :: var_stepc( yp, xp)
!  REAL(KIND=8)       , INTENT(OUT  ) :: var_stepp( xp, yp)
!    CHARACTER(LEN=255) :: PREF1='/scratchout/grupos/dmdpesq/home/luis.goncalves/input/ibis/SA/'
!    CHARACTER(LEN=255) :: PREF1='/data/dmdpesq/SALDAS/DADOS/SALDAS/output/gdas_merge/SURFACEMODEL/'
!    CHARACTER(LEN=255) :: PREF1='/scratch/cptec/luis.goncalves2/work/SALDAS/OUTPUT/SURFACEMODEL/'
!    CHARACTER(LEN=255) :: PREF2='/LIS_HIST_'
!    CHARACTER(LEN=255) :: SUFFIX='00.d01.nc'
!    CHARACTER(LEN=255) :: dat1,dat2,namep

CHARACTER (len = 15), PARAMETER :: vname(8) = (/ &
!	  "Tair_f_inst    ", "Qair_f_inst    ", "Wind_f_inst    ", "Psurf_f_inst   ",&
          "Tair_f_tavg    ", "Qair_f_tavg    ", "SWdown_f_tavg  ", "LWdown_f_tavg  ",&
	  "Wind_f_tavg    ", "Psurf_f_tavg   ", "Rainf_f_tavg   ", "Snowf_tavg     " /)
	  
	  
integer :: i,j,ii,jj,ic

!---------------------------------------
! input and output file names

       !WRITE(namep,'(A8,2(I4.4,3i2.2),A4)')TRIM(PREFIX),&
       !idate(4),idate(2),idate(3),idate(1),&
       !idate(4),idate(2),idate(3),idate(1),&
       !TRIM(SUFFIX)   
        
!       WRITE(dat1, '(I4.4,i2.2)') idatec(4),idatec(2)
!       WRITE(dat2, '(I4.4,3i2.2)') idatec(4),idatec(2),idatec(3),idatec(1)
!       namep=TRIM(PREF1)//TRIM(dat1)//TRIM(PREF2)//TRIM(dat2)//TRIM(SUFFIX)

print *, namep
!RETURN
!file_in  = '/scratchout/grupos/dmdpesq/home/luis.goncalves/input/ibis/SA/200708/LIS_HIST_'//TRIM(ndat)//'00.d01.nc'
file_in = TRIM(namep)
file_out = 'new_file.nc'
!print *,file_in

!---------------------------------------
! Read netcdf input file :

!- open file_in and set its ID value in fidA :
status = NF90_OPEN(TRIM(file_in),0,fidA)
call erreur(status,.TRUE.,"read_A")

!- read ID of dimensions of interest and save them in
!  dimID_TIME, dimID_DEPTH, dimID_LAT, dimID_LON
status = NF90_INQ_DIMID(fidA,"time",dimID_TIME)
call erreur(status,.TRUE.,"inq_dimID_TIME")
!status = NF90_INQ_DIMID(fidA,"DEPTH",dimID_DEPTH)
!call erreur(status,.TRUE.,"inq_dimID_DEPTH")
!status = NF90_INQ_DIMID(fidA,"lat",dimID_LAT)
status = NF90_INQ_DIMID(fidA,"north_south",dimID_LAT)
call erreur(status,.TRUE.,"inq_dimID_LAT")
!status = NF90_INQ_DIMID(fidA,"lon",dimID_LON)
status = NF90_INQ_DIMID(fidA,"east_west",dimID_LON)
call erreur(status,.TRUE.,"inq_dimID_LON")

!- read values of dimensions and store in mTIME, mDEPTH, mLAT, mLON :
status = NF90_INQUIRE_DIMENSION(fidA,dimID_TIME,len=mTIME)
call erreur(status,.TRUE.,"inq_dim_TIME")
!status = NF90_INQUIRE_DIMENSION(fidA,dimID_DEPTH,len=mDEPTH)
!call erreur(status,.TRUE.,"inq_dim_DEPTH")
status = NF90_INQUIRE_DIMENSION(fidA,dimID_LAT,len=mLAT)
call erreur(status,.TRUE.,"inq_dim_LAT")
status = NF90_INQUIRE_DIMENSION(fidA,dimID_LON,len=mLON)
call erreur(status,.TRUE.,"inq_dim_LON")

!- Allocation of arrays :
ALLOCATE(  TIME(mTIME)  )
!ALLOCATE(  DEPTH(mDEPTH)  )
ALLOCATE(  LAT(mLON, mLAT)  )
ALLOCATE(  LON(mLON, mLAT)  )
!ALLOCATE(  TEMP(mLON,mLAT,mDEPTH,mTIME)  )
!ALLOCATE(  TEMP(mLON,mLAT,mTIME)  )
ALLOCATE(  TEMP(mLON,mLAT)  )

print *, mLAT, mLON, mTIME
       


!- read ID of variables of interest and store them in xxxx_ID :
status = NF90_INQ_VARID(fidA,"time",TIME_ID)
call erreur(status,.TRUE.,"inq_TIME_ID")
!status = NF90_INQ_VARID(fidA,"DEPTH",DEPTH_ID)
!call erreur(status,.TRUE.,"inq_DEPTH_ID")
status = NF90_INQ_VARID(fidA,"lat",LAT_ID)
call erreur(status,.TRUE.,"inq_LAT_ID")
status = NF90_INQ_VARID(fidA,"lon",LON_ID)
call erreur(status,.TRUE.,"inq_LON_ID")
!status = NF90_INQ_VARID(fidA,"tems",TEMP_ID)
!status = NF90_INQ_VARID(fidA,"Tair_f_inst",TEMP_ID)
!status = NF90_INQ_VARID(fidA,TRIM(nvar),TEMP_ID)
status = NF90_INQ_VARID(fidA,TRIM(vname(nvar)),TEMP_ID)
call erreur(status,.TRUE.,"inq_TEMP_ID")

!- read and store values of each variable :
status = NF90_GET_VAR(fidA,TIME_ID,TIME)
call erreur(status,.TRUE.,"getvar_TIME")
!status = NF90_GET_VAR(fidA,DEPTH_ID,DEPTH)
!call erreur(status,.TRUE.,"getvar_DEPTH")
status = NF90_GET_VAR(fidA,LAT_ID,LAT, start = (/1, 1/), count = (/mLON, mLAT/))
call erreur(status,.TRUE.,"getvar_LAT")
status = NF90_GET_VAR(fidA,LON_ID,LON, start = (/ 1, 1 /), count = (/ mLON, mLAT /))
call erreur(status,.TRUE.,"getvar_LON")
status = NF90_GET_VAR(fidA,TEMP_ID,TEMP, start = (/ 1, 1 /), count = (/ mLON, mLAT /))
call erreur(status,.TRUE.,"getvar_TEMP")

do i=1,mLON
ic=mLAT
do j=1,mLAT

!if ( TEMP(i,j) .lt. 0.0 ) TEMP(i,j) = 0.0 ! -9.99E+33
!if ( nvar .eq. 1 .AND. TEMP(i,j) .gt. 100 ) TEMP(i,j) = TEMP(i,j) - 273.15
!if ( nvar .eq. 4 .AND. TEMP(i,j) .gt. 100 ) TEMP(i,j) = TEMP(i,j)/100
LAT(i,j)=-55.975+(j-1)*0.05
LON(i,j)=-80.975+(i-1)*0.05

!print *,i,j,LAT(i,j),LON(i,j),TEMP(i,j)

!else
!TEMP(i,j) = 0
!endif
varout(i,j)=TEMP(i,j)
var_stepc(i,j)=TEMP(i,j)

ic=ic-1
enddo
enddo

!    open(10,file='test.bin',status='unknown',access='direct',recl=4)
!      ii=0
!
!     do j=1,mLAT
!     do i=1,mLON
!       ii=ii+1
!       write(10,rec=ii)var_stepc(i,j)
!     enddo
!     enddo
!    CLOSE(10)
!    STOP

print *, TEMP(150,150),var_stepc(150,150)

!- close netcdf file :
status = NF90_CLOSE(fidA)

!latic=LAT
!latip=LAT
!longc=LON
!longp=LON
!var_stepc=varout
!var_stepp=varout

! ii=1
! do i=1,mLON,5
!   jj=1
!   do j=1,mLAT,5
!     latic(ii,jj)=LAT(i,j)
!     latip(ii,jj)=LAT(i,j)
!     longc(ii,jj)=LON(i,j)
!     longp(ii,jj)=LON(i,j)
!     var_stepc(ii,jj)=varout(i,j)
!     var_stepp(ii,jj)=varout(i,j)
!     jj=jj+1
!   enddo
!   ii=ii+1
! enddo

!var_stepc=varout

return
end subroutine modif

!=================================================

SUBROUTINE erreur(iret, lstop, chaine)
! used to provide clear error messages :
USE netcdf
INTEGER, INTENT(in)                     :: iret
LOGICAL, INTENT(in)                     :: lstop
CHARACTER(LEN=*), INTENT(in)            :: chaine
!
CHARACTER(LEN=80)                       :: message
!
IF ( iret .NE. 0 ) THEN
  WRITE(*,*) 'ROUTINE: ', TRIM(chaine)
  WRITE(*,*) 'ERROR: ', iret
  message=NF90_STRERROR(iret)
  WRITE(*,*) 'WHICH MEANS:',TRIM(message)
  IF ( lstop ) STOP
ENDIF
!
END SUBROUTINE erreur


